require "spec_helper"

module Test
  class Engagement
    include Qian::Event
    kafka_topic(:test_topic)

    attribute :user_id, Integer
    attribute :action, String
    attribute :server_time, Integer
    attribute :local_time, Integer
    attribute :placement, Integer
    attribute :props, Hash[String => String]
    attribute :app, String
    attribute :app_version, String
    attribute :real_ip, String
    attribute :batch_id, String
  end
end

describe Qian::Event do
  before(:all) do
    kafka_config = {
        :connection => {
          :seed_brokers => ["10.0.0.1:9092"],
          :client_id => "kafka-client-id"
        },
        :async_config => {
          :delivery_threshold => 100,
          :delivery_interval => 200,
          :max_buffer_size => 300,
          :max_buffer_bytesize => 400
        }
      }

    schema_registry_url = "http://127.0.0.1:28081"

    Qian.configure do |config|
      config.kafka_config = kafka_config
      config.schema_registry_url = schema_registry_url
      config.schema_path = File.join(SPEC_ROOT, "support", "avro_schema")
    end
  end

  describe ".kakfa_topic" do
    it "should correctly set topic name" do
      expect(Test::Engagement.kafka_topic_name).to eq("test_topic")
    end
  end

  describe ".avro_schema_name" do
    it "should return correct avro schema name" do
      expect(Test::Engagement.avro_schema_name).to eq("com.jianshu.event.test.Engagement")
    end
  end

  describe "#avro_encoded_data" do
    it "should return encoded data" do
      engagement_data = {
        "user_id" => 233,
        "action" => "IMPRESSION",
        "server_time" => 10000,
        "local_time" => 20000,
        "props" => {
          "item_type" => "Note",
          "item_id" => "3344"
        },
        "app" => "M7E",
        "app_version" => "4.3.0",
        "placement" => 5,
        "real_ip" => "291.168.1.1",
        "batch_id" => SecureRandom.uuid
      }

      engagement = Test::Engagement.new(engagement_data)

      data = engagement.avro_encoded_data

      expect(data).not_to be_nil
    end
  end

  describe "#emit!" do
    it "should emit event to kafka" do
      engagement_data = {
        "user_id" => 233,
        "action" => "CLICK",
        "server_time" => 10000,
        "local_time" => 20000,
        "props" => {
          "item_type" => "Note",
          "item_id" => "3344"
        },
        "app"=> "HUGO",
        "placement"=> 5,
        "app_version"=> "4.3.0",
        "real_ip"=> "291.168.1.1",
        "batch_id" => SecureRandom.uuid
      }

      engagement = Test::Engagement.new(engagement_data)

      avro_data = "dummy_avro_data"
      expect(engagement).to receive(:avro_encoded_data).and_return(avro_data)

      producer = double
      expect(Qian).to receive(:kafka_producer).and_return(producer)
      expect(producer).to receive(:produce).with(avro_data, :topic => "test_topic")

      engagement.emit!
    end
  end
end
